# Needs packages:
#library(urbanR)
library(gbm)
library(lubridate)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) #per treballar amb cbinds i esas cosas
library(caret)
library(zoo)
library(scatterplot3d)
library(plotly)

#borrar
#library(RNetCDF)
library(sp)
library(sf)
library(mapview)
#library(ncdf4)
#library(rgeos) 
library(ggplot2)
#library(broom)
#library(magrittr)
#library(ggpubr)
library(raster)
#library(rgdal)
#library(tmap)
library(gstat)
#library(abind)
#library(rbind)
library(openair)
#library(tidyr)




############################################################
#                                                          #
#                    Initial data                          #
#                                                          #
############################################################
pollutant       <- "pm2p5"   #no2, o3, pm10
#years           <- c(2020, 2021, 2022)
years           <- 2019
save_result     <-"/home/abarrant/esarchive/scratch/abarrant/results_regional_"   
file_format     <-"gene"   #ghost
time_resolution <-"daily"  #"hourly"
compute_in_UTM  <- "TRUE"
crs_latlon      <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"

path_gbm    <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/GBM_eval/"  ##change
############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
#We will pick the latest data available
local_stations     <- readRDS(paste0(save_result,years[length(years)],"/results_gene_",pollutant, "/output/","local_stations_",file_format, "_",pollutant,".rds"))

############################################################
#                                                          #
#                  Load All observations                   #
#                                                          #
############################################################

df_obs_years <- NULL
for(read_year in years){
  df_obs_all        <- readRDS(file = paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","df_obs_",file_format,"_",pollutant,".rds"))
  
  start_date  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date    <- as.Date(paste(read_year, "-12-31", sep = ""))
  
  dates      <- seq(start_date, end_date, by = "day")                   
  df_obs_sub <- subset(df_obs_all, date>=start_date & date <= end_date) 
  
  
  # Comprovation we don't have duplications in obs
  if(any(duplicated(df_obs_sub$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub$date[duplicated(df_obs_sub$date)]))
    df_obs_sub <- distinct(df_obs_sub, date, .keep_all = TRUE)
    print("The df_obs_sub (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub$date) != length(dates)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub) #combine df by row, filling in missing columns
}

names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_years)))

############################################################
#                                                          #
#            Load raw model interpolated to AQMS           #
#                                                          #
############################################################
df_all_model_raw_at_AQMS <- NULL
for(read_year in years){
  
  model_raw_at_AQMS_df     <- readRDS(paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  df_all_model_raw_at_AQMS <- rbind.fill(df_all_model_raw_at_AQMS, model_raw_at_AQMS_df)
}











# We pick up the observations configured in config file:
obs_filled       <- df_obs_years 
AQMS_obs         <- subset(df_obs_years, select = -date) #we remove the column dates from observations data frame
OK_other_obs_df  <- NULL
list_gbm_results <- NULL
Stats_gbm <- NULL








#recompte mida gaps per totes les estacions

all_nan_counts <- list()
all_availability <- NULL
for (s in 1:length(names_local_stations)){
  
  station                 <- names_local_stations[[s]]
  consecutive_nans <- 0  #inicialitzacio per cada estacio
  nan_counts <- list()
  
  data_training   <- na.omit(AQMS_obs[, station])
  num_nan       <- 365 - length(data_training)
  single_availability <- length(data_training)/365
  all_availability <- c(all_availability, single_availability)
  
  #per cada estacio
  for (i in 1:nrow(AQMS_obs)) {
    
    if (is.na(AQMS_obs[i, station])) {
      #inicio un comptador de nans consecutius 
      consecutive_nans <- consecutive_nans + 1
    } else {
      #si no es nan, pero l'ultim valor era nan, reset del comptador de nans 
      if (consecutive_nans > 0) {
        nan_counts[[length(nan_counts) + 1]] <- consecutive_nans
        consecutive_nans <- 0
      }
    }
  }
  
  #afegir si hi ha nans al final de tot
  if (consecutive_nans > 0) {
    nan_counts[[length(nan_counts) + 1]] <- consecutive_nans
  }
  all_nan_counts <- c(all_nan_counts, nan_counts)
}


print(all_nan_counts)
print(mean(all_availability))

#histogram of all_nan_counts from 2019
all_nan_counts_vector <- unlist(all_nan_counts)

ggplot(data.frame(nan_counts = all_nan_counts_vector), aes(x = nan_counts)) +
  geom_histogram(binwidth = 1, fill = "orange", color = "black") +
  theme_bw() +
  labs(title = "Gap size histogram 2019",
       x = "Observational gap size (days)",
       y = "Frequency") +
  xlim(0, 7)

percentage_x1 <- sum(all_nan_counts_vector == 1)/3935 * 100  #total of missing values 3935
percentage_x2 <- sum(all_nan_counts_vector == 2)/3935 * 100 
percentage_x3 <- sum(all_nan_counts_vector == 3)/3935 * 100 
percentage_x4 <- sum(all_nan_counts_vector == 4)/3935 * 100 
percentage_x5 <- sum(all_nan_counts_vector == 5)/3935 * 100 
percentage_x6 <- sum(all_nan_counts_vector == 6)/3935 * 100 
percentage_x7 <- sum(all_nan_counts_vector == 7)/3935 * 100 

print(paste("Percentage of gap size 1:", percentage_x1))

all_percent <- data_frame(
  gapsize = c(1, 2, 3, 4, 5, 6, 7), 
  percent = c(percentage_x1, percentage_x2, percentage_x3, percentage_x4, percentage_x5, percentage_x6, percentage_x7)
)


ggplot(all_percent, aes(x = gapsize, y = percent, fill = factor(gapsize))) +
  geom_col(binwidth = 1, fill = "orange", color = "black") +
  geom_text(aes(label = round(percent, 1)), vjust = -0.5, color = "black", size = 4) +  # Add text labels on top of the bars
  theme_bw() +
  labs(x = "Observational gap size (consecutive days)",
       y = "Percentatge of occurrence (%)") +
  ylim(0,100) +
  theme(legend.position = "none",
        plot.title = element_text(size = 22),  # Increase title size
        axis.title.x = element_text(size = 22),  # Increase x-axis title size
        axis.title.y = element_text(size = 22),  # Increase y-axis title size
        axis.text.x = element_text(size = 16),  # Increase x-axis text size
        axis.text.y = element_text(size = 16)   # Increase y-axis text size
  ) 










# To fill the gaps of observations in each station, so we need a loop to repeat the fill process in each station:
for (s in 1:length(names_local_stations)){
  
  s <- 13 #eixample
  #s <- 21 #la senia
  station                 <- names_local_stations[[s]]
  
  # Arrangement of AQMS interpolated to AQMS
  # station is the code station that we hidded to interpolate to it, first
  col_station           <- which(station==colnames(df_obs_years))   # index de l'estació en les obs i treure-la
  stations_to_interp    <- df_obs_years[,-col_station]              # obs de les altres estacions sense l'estació a interpolar
  c <- 0
  #Loop over all days to interpolate data from other station to station
  for(j in 1:length(df_obs_years$date)){
    #j <- 1
    #print(paste0("Date: ", stations_to_interp$date[j]))
    #We take the data day by day /hour by hour
    ok_obs    <- stations_to_interp[j,] #obs per dia de totes les estacions menys la eliminada
    ok_obs    <- subset(ok_obs,select=-date)
    df_ok_obs <- data.frame(ref_obs = colnames(ok_obs), pollutant = as.numeric(ok_obs)) #data frame estacions x 1 dia
    
    #coordinates where to interpolate:
    df_hidded_AQMS <- local_stations[ which(local_stations$ref_obs == station) , ] #spatial PDF de l'estació a interpolar
    
    if (compute_in_UTM) {
      df_hidded_AQMS <- sp::spTransform(df_hidded_AQMS, sp::CRS(crs_utm))
    }
    
    #    1.2 Compute Variogram
    #with other AQMS we want a SPDF that has the ref of the station and the coordinates of each, with the value of the pollution in that day/hour
    sp_other_ok_obs <- local_stations[ -which(local_stations$ref_obs == station) , ] #spatial PDF on hi han totes les coords excepte l'amagada (sobren estacions)
    df_other_obs <- as.data.frame(sp_other_ok_obs) #pasem de spdf a df per poder filtrar
    df_obs_data  <- merge(df_other_obs[, -which(names(df_other_obs) == "pollutant")], df_ok_obs, by = "ref_obs") #copiem les coordenades de les estacions d'interès amb les observacion que pertoquen
    
    if(!sum(is.na(df_obs_data$pollutant))>=length(df_obs_data$pollutant)){
      df_obs_data                   <- na.omit(df_obs_data)
      coordinates(df_obs_data)      <- ~lon+lat
      sp::proj4string(df_obs_data)  <- crs_latlon
      
      if (compute_in_UTM) {
        df_obs_data <- spTransform(df_obs_data, sp::CRS(crs_utm))
      }
      
      #efectuem l'ordinary kriging
      pollutant.fit_ok          <- automap::autofitVariogram(formula=pollutant~1, input_data=df_obs_data,model="Ste")  #stein again funciona millor per pm2p5
      
      ok_gstat <- gstat(formula=pollutant~1, data = df_obs_data, model = pollutant.fit_ok$var_model, nmax=100)
      OK       <- predict(object=ok_gstat, newdata=df_hidded_AQMS)
      
      
      OK_other_obs           <- data.frame(data=OK$var1.pred)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      
    } else{
      #print(paste0("There is no observations in: ", stations_to_interp$date[j]))
      OK_other_obs           <- data.frame(data=NA)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df        <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      c <- c+1
    }
    
  }
  print(paste0("The number of days without any observational data is: ", c))  #dies pels quals no hi ha cap obs a cap de les estacions i doncs no pot interpolar sense dades
  
  # Select station data (PREDICTORS)
  single_station_data <- data.frame(date            = OK_other_obs_df$date,
                                    obs             = AQMS_obs[,station],                     #valor obs a l'estacio (de vegades nan)
                                    model           = df_all_model_raw_at_AQMS[,station],     #valor donat pel model a l'estacio
                                    day_of_year     = yday(OK_other_obs_df$date),             #julian day
                                    day_of_week     = wday(OK_other_obs_df$date),
                                    other_obs       = OK_other_obs_df[,station])              #predictor del valor interpolat a lestacio a partir de totes les altres 
  
  single_station_data <- na.omit(single_station_data)
  
  
}


# PLOTS OBSERVATIONAL DATA -----------------------------------------------------

#eixample <- single_station_data
save_path      <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
eixample  <- readRDS(file=paste0(save_path, "eixample_OK_vs_OBS.rds"))

ggplot(eixample, aes(x = obs, y = other_obs)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "black", size=0.5, linetype = "dashed") +
  theme_bw() +
  labs(title = "Eixample (r = 0.57)",
       x = expression("Observational PM2.5 (" * mu * "g/m"^3 * ")"),
       y = expression("Interpolated PM2.5 (" * mu * "g/m"^3 * ")")) +
  ylim(5,40) +
  xlim(5,55) +
  theme(legend.position = "none",
        plot.title = element_text(size = 23),  # Increase title size
        axis.title.x = element_text(size = 23),  # Increase x-axis title size
        axis.title.y = element_text(size = 23),  # Increase y-axis title size
        axis.text.x = element_text(size = 16),  # Increase x-axis text size
        axis.text.y = element_text(size = 16)   # Increase y-axis text size
  ) 


Summary_stats_eix <- openair::modStats(eixample, obs = "obs", mod = "other_obs")
#saveRDS(object=Summary_stats_eix, file=paste0(save_path,"Summary_stats_eix.rds"))

#save_path      <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
#saveRDS(object=eixample, file=paste0(save_path, "eixample_OK_vs_OBS.rds"))


#senia <- single_station_data
save_path      <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
senia <- readRDS(file=paste0(save_path, "senia_OK_vs_OBS.rds"))

ggplot(senia, aes(x = obs, y = other_obs)) +
  geom_point() +
  geom_smooth(method = "lm", se = FALSE, color = "black", size=0.5, linetype = "dashed") +
  theme_bw() +
  labs(title = "La Sènia (r = 0.25)",
       x = expression("Observational PM2.5 (" * mu * "g/m"^3 * ")"),
       y = expression("Interpolated PM2.5 (" * mu * "g/m"^3 * ")")) +
  ylim(0,40) +
  xlim(0,17) +
  theme(legend.position = "none",
        plot.title = element_text(size = 23),  # Increase title size
        axis.title.x = element_text(size = 23),  # Increase x-axis title size
        axis.title.y = element_text(size = 23),  # Increase y-axis title size
        axis.text.x = element_text(size = 16),  # Increase x-axis text size
        axis.text.y = element_text(size = 16)   # Increase y-axis text size
  ) 
  

Summary_stats_senia <- openair::modStats(senia, obs = "obs", mod = "other_obs")
#saveRDS(object=Summary_stats_senia, file=paste0(save_path,"Summary_stats_senia.rds"))

#save_path      <- "/home/abarrant/esarchive/scratch/abarrant/results_regional_2019/results_gene_pm2p5/stats_results/"
#saveRDS(object=senia, file=paste0(save_path, "senia_OK_vs_OBS.rds"))

  