library(startR)


# A path pattern is built
repos <- '/esarchive/scratch/ccarnere/AQS_database/postprocessed/regional/hourly/$var$/$var$_$sdate$.nc'

# A Start() call is built with the indices to operate
data <- Start(dat = repos,
              var = 'sconcno2',
              sdate = '2017010100',
              ensemble = 'all',
              time = 'all',
              latitude = 'all',
              longitude = 'all', 
              retrieve = FALSE)

#cas nostre
easyNCDF::NcReadDims('/esarchive/scratch/abarrant/test_netcdf/sconcno2/2017/sconcno2_2017010100.nc',
                     var_names = 'sconcno2')

#cas exemple
easyNCDF::NcReadDims('/esarchive/exp/ecmwf/system5_m1/6hourly/tas/tas_19930101.nc',
                     var_names = 'tas')



fun <- function(x) {
  # Expected inputs:
  #   x: array with dimensions ('ensemble', 'time')
  apply(x + 1, 2, mean)
}

step <- Step(fun, 
             target_dims = c('ensemble', 'time'), 
             output_dims = c('time'))

wf <- AddStep(data, step)

res <- Compute(wf,
               chunks = list(latitude = 2,
                             longitude = 2),
               threads_load = 2,
               threads_compute = 4,
               cluster = list(queue_host = 'nord4.bsc.es',
                              queue_type = 'slurm',
                              temp_dir = '/gpfs/scratch/bsc32/bsc32734/startR_hpc/',
                              cores_per_job = 4,
                              job_wallclock = '00:10:00',
                              max_jobs = 4,
                              extra_queue_params = list('#SBATCH --constraint=medmem'),
                              bidirectional = FALSE,
                              polling_period = 10
               ),
               ecflow_suite_dir = '/home/Earth/aho/startR_local/',
               wait = TRUE
)
