library(gbm)
library(lubridate)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr) #per treballar amb cbinds i esas cosas
library(caret)
library(zoo)
library(scatterplot3d)
library(plotly)

#borrar
#library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
#library(ncdf4)
#library(rgeos) 
library(ggplot2)
#library(broom)
#library(magrittr)
#library(ggpubr)
#library(raster)
#library(rgdal)
#library(tmap)
library(gstat)
#library(abind)
#library(rbind)
library(openair)
#library(tidyr)



############################################################
#                                                          #
#                    Initial data                          #
#                                                          #
############################################################
pollutant       <- "pm2p5"   #no2, o3, pm10
#years           <- c(2020, 2021, 2022)
years           <- 2019
save_result     <-"/esarchive/scratch/abarrant/results_regional_"   
file_format     <-"gene"   #ghost
time_resolution <-"daily"  #"hourly"
compute_in_UTM  <- "TRUE"
crs_latlon      <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0"
crs_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"

path_gbm    <- "/esarchive/scratch/abarrant/results_regional_2019/GBM_eval/"  ##change
############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
#We will pick the latest data available
local_stations     <- readRDS(paste0(save_result,years[length(years)],"/results_gene_",pollutant, "/output/","local_stations_",file_format, "_",pollutant,".rds"))

############################################################
#                                                          #
#                  Load All observations                   #
#                                                          #
############################################################

df_obs_years <- NULL
for(read_year in years){
  df_obs_all        <- readRDS(file = paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","df_obs_",file_format,"_",pollutant,".rds"))
  
  start_date  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date    <- as.Date(paste(read_year, "-12-31", sep = ""))
  
  dates      <- seq(start_date, end_date, by = "day")                   
  df_obs_sub <- subset(df_obs_all, date>=start_date & date <= end_date) 
  
  
  # Comprovation we don't have duplications in obs
  if(any(duplicated(df_obs_sub$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub$date[duplicated(df_obs_sub$date)]))
    df_obs_sub <- distinct(df_obs_sub, date, .keep_all = TRUE)
    print("The df_obs_sub (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub$date) != length(dates)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub) #combine df by row, filling in missing columns
}

names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_years)))

############################################################
#                                                          #
#            Load raw model interpolated to AQMS           #
#                                                          #
############################################################
df_all_model_raw_at_AQMS <- NULL
for(read_year in years){
  
  model_raw_at_AQMS_df     <- readRDS(paste0(save_result,read_year,"/results_gene_",pollutant, "/output/","model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  df_all_model_raw_at_AQMS <- rbind.fill(df_all_model_raw_at_AQMS, model_raw_at_AQMS_df)
}
